import finconfig
import finmatcher
import datetime
import calendar
from pyparsing import *

class finDateRelationMatcher(finmatcher.finMatcher):
  """Matches a date relation given in a script line"""
  _relops={
    '<': finconfig.lt,
    '<=': finconfig.le,
    '>': finconfig.gt,
    '>=': finconfig.ge,
    '==': finconfig.eq,
    '!=': finconfig.ne
  }
  _units={
    's': 1,
    'm': 4,
    'h': 5,
    'd': 0,
    'w': 6
  }
  _bnf=Word('0123456789') + Regex('|'.join(_units))

  def match(self, input):
    return self._op(self._abs, input)

  def parse(self, rel, input):
    if rel not in self._relops:
      raise fincoretypes.finInternalParseError('Invalid relop found', rel)
    spl=self._bnf.parseString(input)
    self._td=datetime.timedelta(*([0]*self._units[spl[1]]+[int(spl[0])]))
    self._op=self._relops[rel]
    self._abs=calendar.timegm((datetime.datetime.utcnow() - \
      self._td).utctimetuple())
