from pyparsing import *
import fincorestrings

class finSegment(object):
  """Composes a segment of fin's workflow
Other segment types are expected to derive from this class and override
or extend the following attributes and methods:

  - inputs
  - outputs
  - outputforinput()
  - keyword
  - __iter__()  
  """
  arguments={}
  mandatoryarguments={}
  
  inputs=None
  """A list of the types the segment accepts from the previous segment"""

  outputs=None
  """A list of the types the segment outputs to the next segment. If None,
  then call .outputforinput() with the type of the output from the previous
  segment"""

  keyword=None
  """The keyword used to identify this segment at the beginning of
  a script line"""

  _args={}

  def __init__(self, previous):
    self._allargs=dict(self.arguments)
    self._allargs.update(self.mandatoryarguments)
    self._prev=previous

  def outputforinput(self, intype):
    """Returns the type output by the segment for a given input type"""
    pass

  def parse(self, input):
    """Parses a script line for the current segment"""
    for i in input:
#      print 'Arg:', i
      t=self._allargs[i[0]]()
      if isinstance(t, fincorestrings.finArgument):
        self._args[i[0]]=t.parse(i[1], i[2])
      else:
        self._args[i[0]]=t
        t.parse(i[1], i[2])

  def __iter__(self):
    pass