import finconfig
import fincoretypes
import finmatcher
import re
import fnmatch
from pyparsing import *

class finBasicStringStrategy(object):
  """Matches plain strings. Doesn't store the pattern since this
  is meant to be a lightweight object"""
  def __init__(self):
    self._value=''

  def match(self, input, value):
    """Matches the 2 values passed"""
    if input==value:
      return input

  def parse(self, input, flags):
    """Parses the pattern passed"""
    return input.decode('unicode_escape')

class finRegexStringStrategy(finBasicStringStrategy):
  """Matches strings to a regular expression"""
  def match(self, input, value):
    """Matches the 2 values passed"""
    if value.match(input):
      return input

  def parse(self, input, flags):
    """Parses the pattern passed"""
    return re.compile(input.decode('unicode_escape'),
      sum(getattr(re, x) for x in flags))

class finGlobStringStrategy(finBasicStringStrategy):
  def match(self, input, value):
    """Matches the 2 values passed"""
    if fnmatch.fnmatchcase(input, value):
      return input

class finRawStringStrategy(finBasicStringStrategy):
  def parse(self, input, flags):
    """Parses the pattern passed"""
    return input

class finStringMatcher(finmatcher.finMatcher):
  """Matches a string pattern given in a script line"""  
  _strats={
    u'': finBasicStringStrategy,
    u're': finRegexStringStrategy,
    u'glob': finGlobStringStrategy,
    u'raw': finRawStringStrategy
  }
  types={
    u'String': u'',
    u'Regular expression': u're',
    u'Globbed expression': u'glob',
    u'Raw string': u'raw'
  }
  _stratmax=len(max(_strats, key=len))
  _bnf=oneOf(_strats.keys()) + \
    quotedString.copy().setParseAction(removeQuotes) + \
    Optional(Literal(u'/').suppress() + \
      OneOrMore(oneOf(u'I L M S U')))\
      .setParseAction(lambda x: u''.join(x)) + \
    restOfLine

  def __init__(self):
    self._strat=finBasicStringStrategy()

  def match(self, input):
    """Match the pattern with a string inside the segment"""
    return self._strat.match(input, self._value)

  def parse(self, rel, input):
    """Parse the string pattern as passed from the script line"""  
    if rel!=u'=':
      raise fincoretypes.finInternalParseException(u'Invalid relop found', \
      rel)
    spl=self._bnf.parseString(input)
#    print 'spl:', spl
    strat=self._strats.get(spl[0])
    if strat is None:
      raise fincoretypes.finInternalParseException(u'Invalid string type '\
      'found', spl[0])
    self._strat=strat()
    try:
      self._value=self._strat.parse(spl[1], spl[2])
      return spl[1]
    except ParseException,e:
      raise finInternalParseException(u'Bad string format', e)
      
class finBasicStringMatcher(finStringMatcher):
  """Matches a string pattern given in a script line"""  
  _strats={
    u'': finBasicStringStrategy,
  }
  _bnf=Regex('') + quotedString.copy().setParseAction(removeQuotes) + \
    Regex('') + Regex('.*')
    
class finArgument(finBasicStringMatcher):
  _strats={
    u'': finBasicStringStrategy,
  }
